// This package contains functions that are useful to generate
// palyndrome strings at random.
package gen

import "math/rand"

// randLetter returns a random character between 'a' and 'z'
func randLetter() rune {
	k := rand.Intn(26)
	return rune('a' + k)
}

// isVowel given a character c which is assumed to be a lowercase Latin letter,
// determines if it is a vowel
func isVowel(c rune) bool {
	return c == 'a' || c == 'e' || c == 'i' || c == 'o' || c == 'u' || c == 'y'
}

// returns a random vowel
func randVowel() rune {
	for {
		c := randLetter()
		if isVowel(c) {
			return c
		}
	}
}

// returns a random consonant
func randConsonant() rune {
	for {
		c := randLetter()
		if !isVowel(c) {
			return c
		}
	}
}

// GenPalyndromIterative generates a palyndrome string of length n.
// The characters will only be lowercase Latin letters.
func GenPalyndromeIterative(n int) string {
	var res string
	if n % 2 == 0 {
		res = ""
	} else {
		res = string(randLetter())
	}
	for len(res) < n {
		c := randLetter()
		res = string(c) + res + string(c)
	}
	return res
}


// GenPalyndromIterativeReadable generates a palyndrome string of length n.
// The characters will only be lowercase Latin letters, alternating between consonants and
// vowels.
func GenPalyndromeIterativeReadable(n int) string {
	var res string
	var lastIsVowel bool
	var c rune
	if n % 2 == 0 {
		res = ""
	} else {
		res = string(randConsonant())
	}
	for len(res) < n {
		if lastIsVowel {
			c = randConsonant()
			lastIsVowel = false
		} else {
			c = randVowel()
			lastIsVowel = true
		}
		res = string(c) + res + string(c)
	}
	return res
}

// GenPalyndromRecursive generates a palyndrome string of length n.
// The characters will only be lowercase Latin letters.
func GenPalyndromeRecursive(n int) string {
	if n == 0 {
		return ""
	}
	if n == 1 {
		return string(randLetter())
	}
	c := randLetter()
	s := GenPalyndromeRecursive(n - 2)
	return string(c) + s + string(c)
}

